package com.zhongshu.reward.server.core.util;

import cn.hutool.log.Log;
import lombok.extern.slf4j.Slf4j;

import java.security.SecureRandom;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.util.Base64;
import java.util.Calendar;
import java.util.Date;

/**
 * @author wjf
 * @date 2024/8/8
 */
@Slf4j
public class DateUtils {

    public final static String FORMAT_LONG = "yyyy-MM-dd HH:mm:ss";
    public static Long timeToLong(String time, String format) {
        if (!org.springframework.util.StringUtils.hasText(time)) {
            return null;
        }
        try {
            return new SimpleDateFormat(format).parse(time).getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     * 获得本天的开始时间
     *
     * @return
     */
    public static Date getCurrentDayStartTime() {
        ZoneId zoneId = ZoneId.systemDefault();
        LocalDateTime date = LocalDateTime.now();
        LocalDateTime startOfTheDay = LocalDateTime.of(date.toLocalDate(), LocalTime.MIN);
        Date beginTime = Date.from(startOfTheDay.atZone(zoneId).toInstant());
        return beginTime;
    }

    /**
     * 获得本天的结束时间
     *
     * @return
     */
    public static Date getCurrentDayEndTime() {
        ZoneId zoneId = ZoneId.systemDefault();
        LocalDateTime date = LocalDateTime.now();
        LocalDateTime startOfTheDay = LocalDateTime.of(date.toLocalDate(), LocalTime.MAX);
        Date beginTime = Date.from(startOfTheDay.atZone(zoneId).toInstant());
        return beginTime;
    }

    /**
     * 次月指定日开始时间
     *
     * @return
     */
    public static Long nextMonthDayStartTime(Integer day) {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, 1);

        if (day >c.getActualMaximum(Calendar.DAY_OF_MONTH)){
            c.set(Calendar.DATE, c.getActualMaximum(Calendar.DAY_OF_MONTH));
        }else {
            c.set(Calendar.DATE, day);
        }
        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        return c.getTimeInMillis();
    }

    /**
     * 次月指定日开始时间
     *
     * @return
     */
    public static Long lastMonthDayStartTime(Integer day) {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, -1);

        if (day >c.getActualMaximum(Calendar.DAY_OF_MONTH)){
            c.set(Calendar.DATE, c.getActualMaximum(Calendar.DAY_OF_MONTH));
        }else {
            c.set(Calendar.DATE, day);
        }
        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        return c.getTimeInMillis();
    }

    /**
     * 上月开始时间
     *
     * @return
     */
    public static Long lastMonthStartTime() {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, -1);
        c.set(Calendar.DATE, 1);

        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        return c.getTimeInMillis();
    }

    /**
     * 上月结束时间
     *
     * @return
     */
    public static Long lastMonthEndTime() {
        Calendar c = Calendar.getInstance();

        c.add(Calendar.MONTH, 0);
        c.set(Calendar.DATE, 1);
        c.set(Calendar.DAY_OF_MONTH, 0);

        c.set(Calendar.HOUR_OF_DAY, 23);
        c.set(Calendar.MINUTE, 59);
        c.set(Calendar.SECOND, 59);

        return c.getTimeInMillis();
    }

    /**
     * 本月指定日的开始时间
     *
     * @return
     */
    public static Long getDayOfMonthEndTime(Integer day) {
        Calendar c = Calendar.getInstance();

        c.set(Calendar.DATE, 1);
        c.add(Calendar.DATE, day-1);

        c.set(Calendar.HOUR_OF_DAY, 23);
        c.set(Calendar.MINUTE, 59);
        c.set(Calendar.SECOND, 59);

        return c.getTimeInMillis();
    }

    /**
     * 本月指定日的结束时间
     *
     * @return
     */
    public static Long getDayOfMonthStartTime(Integer day) {
        Calendar c = Calendar.getInstance();

        c.set(Calendar.DATE, 1);
        c.add(Calendar.DATE, day-1);

        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);

        return c.getTimeInMillis();
    }

    /**
     * 指定年月开始时间
     *
     * @return
     */
    public static Long getMonthStartTime(Integer year, Integer month) {
        Calendar c = Calendar.getInstance();

        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, month - 1);
        c.set(Calendar.DATE, 1);

        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);

        return c.getTimeInMillis();

    }

    /**
     * 指定年月结束时间
     *
     * @return
     */
    public static Long getMonthEndTime(Integer year, Integer month) {
        Calendar c = Calendar.getInstance();

        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, month - 1);
        c.add(Calendar.MONTH, 1);
        c.set(Calendar.DAY_OF_MONTH, 0);

        c.set(Calendar.HOUR_OF_DAY, 23);
        c.set(Calendar.MINUTE, 59);
        c.set(Calendar.SECOND, 59);
        return c.getTimeInMillis();
    }

    /**
     * 当天指定时分开始时间 time: HH:mm
     *
     * @return
     */
    public static Long getHourMinuteStartTime(Integer hour, Integer minute) {
        Calendar c = Calendar.getInstance();

        c.set(Calendar.HOUR_OF_DAY, hour);
        c.set(Calendar.MINUTE, minute);
        c.set(Calendar.SECOND, 59);
        return c.getTimeInMillis();
    }

    /**
     * 当天指定时分结束时间 time: HH:mm
     *
     * @return
     */
    public static Long getHourMinuteEndTime(Integer hour, Integer minute) {
        Calendar c = Calendar.getInstance();

        c.set(Calendar.HOUR_OF_DAY, 21);
        c.set(Calendar.MINUTE, hour);
        c.set(Calendar.SECOND, minute);
        return c.getTimeInMillis();
    }

    public static String paresTime(Long time, String pattern) {
        if (time == null || time <= 0) {
            return "";
        }
        Date date = new Date(time);
        SimpleDateFormat formatter = new SimpleDateFormat(pattern);
        return formatter.format(date);
    }

    public static void main(String[] args) {
        SecureRandom secureRandom = new SecureRandom();
        byte[] key = new byte[32];
        secureRandom.nextBytes(key);
        ;
        log.info("密钥：{}", Base64.getEncoder().encodeToString(key).substring(0, key.length));
        log.info(paresTime(getDayOfMonthStartTime(2), FORMAT_LONG));
        log.info(paresTime(getDayOfMonthEndTime(2), FORMAT_LONG));
    }
}
